# -*- cperl -*-
# copyright (C) 2005 Topia <topia@clovery.jp>. all rights reserved.
# This is free software; you can redistribute it and/or modify it
#   under the same terms as Perl itself.
# $Id: ListFriend.pm 30 2005-01-29 20:50:35Z topia $
# $URL: file:///usr/minetools/svnroot/mixi/trunk/WWW-Mixi-OO/lib/WWW/Mixi/OO/ListFriend.pm $
package WWW::Mixi::OO::ListFriend;
use strict;
use warnings;
use base qw(WWW::Mixi::OO::TablePreviewListPage);

1;
